@@ -65,6 +65,7 @@
 #include "repo_read.h"
 #include "repo_write.h"
 #include "notify.h"
+#include "secrets.h"
 
 #ifndef nitems
 #define nitems(_a)	(sizeof((_a)) / sizeof((_a)[0]))
@@ -126,7 +127,8 @@ static void drop_privs(struct passwd *);
 __dead static void
 usage(void)
 {
-	fprintf(stderr, "usage: %s [-dnv] [-f config-file]\n", getprogname());
+	fprintf(stderr, "usage: %s [-dnv] [-f config-file] [-s secrets]\n",
+	    getprogname());
 	exit(1);
 }
 
@@ -1656,9 +1658,9 @@ static pid_t
 start_child(enum gotd_procid proc_id, const char *repo_path,
     char *argv0, const char *confpath, int fd, int daemonize, int verbosity)
 {
-	char	*argv[11];
-	int	 argc = 0;
-	pid_t	 pid;
+	const char	*argv[11];
+	int		 argc = 0;
+	pid_t		 pid;
 
 	switch (pid = fork()) {
 	case -1:
@@ -1679,47 +1681,47 @@ start_child(enum gotd_procid proc_id, const char *repo_path,
 	argv[argc++] = argv0;
 	switch (proc_id) {
 	case PROC_LISTEN:
-		argv[argc++] = (char *)"-L";
+		argv[argc++] = "-TL";
 		break;
 	case PROC_AUTH:
-		argv[argc++] = (char *)"-A";
+		argv[argc++] = "-TA";
 		break;
 	case PROC_SESSION_READ:
-		argv[argc++] = (char *)"-s";
+		argv[argc++] = "-Ts";
 		break;
 	case PROC_SESSION_WRITE:
-		argv[argc++] = (char *)"-S";
+		argv[argc++] = "-TS";
 		break;
 	case PROC_REPO_READ:
-		argv[argc++] = (char *)"-R";
+		argv[argc++] = "-TR";
 		break;
 	case PROC_REPO_WRITE:
-		argv[argc++] = (char *)"-W";
+		argv[argc++] = "-TW";
 		break;
 	case PROC_NOTIFY:
-		argv[argc++] = (char *)"-N";
+		argv[argc++] = "-TN";
 		break;
 	default:
 		fatalx("invalid process id %d", proc_id);
 	}
 
-	argv[argc++] = (char *)"-f";
-	argv[argc++] = (char *)confpath;
+	argv[argc++] = "-f";
+	argv[argc++] = confpath;
 
 	if (repo_path) {
-		argv[argc++] = (char *)"-P";
-		argv[argc++] = (char *)repo_path;
+		argv[argc++] = "-P";
+		argv[argc++] = repo_path;
 	}
 
 	if (!daemonize)
-		argv[argc++] = (char *)"-d";
+		argv[argc++] = "-d";
 	if (verbosity > 0)
-		argv[argc++] = (char *)"-v";
+		argv[argc++] = "-v";
 	if (verbosity > 1)
-		argv[argc++] = (char *)"-v";
+		argv[argc++] = "-v";
 	argv[argc++] = NULL;
 
-	execvp(argv0, argv);
+	execvp(argv0, (char * const *)argv);
 	fatal("execvp");
 }
 
@@ -2049,12 +2051,34 @@ unveil_notification_helpers(void)
 		fatal("unveil");
 }
 
+static void
+check_file_secrecy(int fd, const char *fname)
+{
+	struct stat st;
+
+	if (fstat(fd, &st))
+		fatal("cannot stat %s", fname);
+
+	if (st.st_uid != 0)
+		fatalx("secrets file %s must be owned by root", fname);
+
+	if (st.st_gid != 0)
+		fatalx("secrets file %s must be owned by group wheel/root",
+		    fname);
+
+	if (st.st_mode & (S_IWGRP | S_IXGRP | S_IRWXO))
+		fatalx("secrets file %s must not be group writable or world "
+		    "readable/writable", fname);
+}
+
 int
 main(int argc, char **argv)
 {
 	const struct got_error *error = NULL;
+	struct gotd_secrets *secrets = NULL;
 	int ch, fd = -1, daemonize = 1, verbosity = 0, noaction = 0;
 	const char *confpath = GOTD_CONF_PATH;
+	char *secretspath = NULL;
 	char *argv0 = argv[0];
 	char title[2048];
 	struct passwd *pw = NULL;
@@ -2066,6 +2090,7 @@ main(int argc, char **argv)
 	struct gotd_repo *repo = NULL;
 	char *default_sender = NULL;
 	char hostname[_POSIX_HOST_NAME_MAX + 1];
+	FILE *fp;
 	FILE *diff_f1 = NULL, *diff_f2 = NULL;
 	int diff_fd1 = -1, diff_fd2 = -1;
 	const char *errstr;
@@ -2074,47 +2099,58 @@ main(int argc, char **argv)
 
 	log_init(1, LOG_DAEMON); /* Log to stderr until daemonized. */
 
-	while ((ch = getopt(argc, argv, "Adf:LnNP:RsSvW")) != -1) {
+	while ((ch = getopt(argc, argv, "df:nP:s:T:v")) != -1) {
 		switch (ch) {
-		case 'A':
-			proc_id = PROC_AUTH;
-			break;
 		case 'd':
 			daemonize = 0;
 			break;
 		case 'f':
 			confpath = optarg;
 			break;
-		case 'L':
-			proc_id = PROC_LISTEN;
-			break;
 		case 'n':
 			noaction = 1;
 			break;
-		case 'N':
-			proc_id = PROC_NOTIFY;
-			break;
 		case 'P':
 			repo_path = realpath(optarg, NULL);
 			if (repo_path == NULL)
 				fatal("realpath '%s'", optarg);
 			break;
-		case 'R':
-			proc_id = PROC_REPO_READ;
-			break;
 		case 's':
-			proc_id = PROC_SESSION_READ;
+			secretspath = realpath(optarg, NULL);
+			if (secretspath == NULL)
+				fatal("realpath '%s'", optarg);
 			break;
-		case 'S':
-			proc_id = PROC_SESSION_WRITE;
+		case 'T':
+			switch (*optarg) {
+			case 'A':
+				proc_id = PROC_AUTH;
+				break;
+			case 'L':
+				proc_id = PROC_LISTEN;
+				break;
+			case 'N':
+				proc_id = PROC_NOTIFY;
+				break;
+			case 'R':
+				proc_id = PROC_REPO_READ;
+				break;
+			case 's':
+				proc_id = PROC_SESSION_READ;
+				break;
+			case 'S':
+				proc_id = PROC_SESSION_WRITE;
+				break;
+			case 'W':
+				proc_id = PROC_REPO_WRITE;
+				break;
+			default:
+				errx(1, "unknown proc type %s", optarg);
+			}
 			break;
 		case 'v':
 			if (verbosity < 3)
 				verbosity++;
 			break;
-		case 'W':
-			proc_id = PROC_REPO_WRITE;
-			break;
 		default:
 			usage();
 		}
@@ -2129,7 +2165,24 @@ main(int argc, char **argv)
 	if (geteuid() && (proc_id == PROC_GOTD || proc_id == PROC_LISTEN))
 		fatalx("need root privileges");
 
-	if (parse_config(confpath, proc_id, &gotd) != 0)
+	if (proc_id == PROC_GOTD) {
+		const char *p = secretspath ? secretspath : GOTD_SECRETS_PATH;
+
+		fp = fopen(p, "r");
+		if (fp == NULL && (secretspath != NULL || errno != ENOENT))
+			fatal("can't open secrets file %s", p);
+
+		if (fp != NULL) {
+			check_file_secrecy(fileno(fp), p);
+			error = gotd_secrets_parse(p, fp, &secrets);
+			fclose(fp);
+			if (error)
+				fatalx("failed to parse secrets file %s: %s",
+				    p, error->msg);
+		}
+	}
+
+	if (parse_config(confpath, proc_id, secrets, &gotd) != 0)
 		return 1;
 
 	pw = getpwnam(gotd.user_name);
@@ -2424,12 +2477,57 @@ main(int argc, char **argv)
 	signal_add(&evsigchld, NULL);
 
 	gotd_imsg_event_add(&gotd.listen_proc->iev);
-	if (gotd.notify_proc)
+	if (gotd.notify_proc) {
+		struct imsgbuf *imsgbuf = &gotd.notify_proc->iev.ibuf;
+		struct gotd_secret *s;
+		size_t i, n = 0;
+
 		gotd_imsg_event_add(&gotd.notify_proc->iev);
 
+		if (gotd.secrets)
+			n = gotd.secrets->len;
+
+		if (imsg_compose(imsgbuf, GOTD_IMSG_SECRETS, 0, 0, -1,
+		    &n, sizeof(n)) == -1)
+			fatal("imsg_compose GOTD_IMSG_SECRETS");
+		if (imsg_flush(imsgbuf))
+			fatal("imsg_flush");
+
+		for (i = 0; i < n; ++i) {
+			struct iovec iov[5];
+
+			s = &gotd.secrets->secrets[i];
+
+			iov[0].iov_base = &s->type;
+			iov[0].iov_len = sizeof(s->type);
+
+			iov[1].iov_base = s->label;
+			iov[1].iov_len = strlen(s->label) + 1;
+
+			iov[2].iov_base = s->user;
+			iov[2].iov_len = s->user ? strlen(s->user) + 1 : 0 ;
+
+			iov[3].iov_base = s->pass;
+			iov[3].iov_len = s->pass ? strlen(s->pass) + 1 : 0 ;
+
+			iov[4].iov_base = s->hmac;
+			iov[4].iov_len = s->hmac ? strlen(s->hmac) + 1 : 0 ;
+
+			if (imsg_composev(imsgbuf, GOTD_IMSG_SECRET,
+			    0, 0, -1, iov, 5) == -1)
+				fatal("imsg_composev GOTD_IMSG_SECRET");
+			if (imsg_flush(imsgbuf))
+				fatal("imsg_flush");
+		}
+
+		gotd_secrets_free(gotd.secrets);
+		gotd.secrets = NULL;
+	}
+
 	event_dispatch();
 
 	free(repo_path);
+	free(secretspath);
 	free(default_sender);
 	gotd_shutdown();
 
