@@ -250,14 +250,9 @@ list_refs(struct imsg *imsg)
 	size_t datalen;
 	struct gotd_imsg_reflist irefs;
 	struct imsgbuf ibuf;
-	int client_fd;
 
 	TAILQ_INIT(&refs);
 
-	client_fd = imsg_get_fd(imsg);
-	if (client_fd == -1)
-		return got_error(GOT_ERR_PRIVSEP_NO_FD);
-
 	datalen = imsg->hdr.len - IMSG_HEADER_SIZE;
 	if (datalen != 0)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
@@ -268,13 +263,16 @@ list_refs(struct imsg *imsg)
 	}
 	repo_write.refs_listed = 1;
 
-	client->fd = client_fd;
+	client->fd = imsg_get_fd(imsg);
+	if (client->fd == -1)
+		return got_error(GOT_ERR_PRIVSEP_NO_FD);
+
 	client->nref_updates = 0;
 	client->nref_del = 0;
 	client->nref_new = 0;
 	client->nref_move = 0;
 
-	imsg_init(&ibuf, client_fd);
+	imsg_init(&ibuf, client->fd);
 
 	err = got_ref_list(&refs, repo_write.repo, "",
 	    got_ref_cmp_by_name, NULL);
@@ -1240,7 +1238,7 @@ recv_packfile(int *have_packfile, struct imsg *imsg)
 
 	err = got_delta_cache_alloc(&pack->delta_cache);
 	if (err)
-		return err;
+		goto done;
 
 	for (i = 0; i < nitems(repo_tempfiles); i++) {
 		struct repo_tempfile *t = &repo_tempfiles[i];
@@ -2330,19 +2328,26 @@ repo_write_dispatch(int fd, short event, void *arg)
 	if (event & EV_READ) {
 		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
 			fatal("imsg_read error");
-		if (n == 0)	/* Connection closed. */
+		if (n == 0) {	/* Connection closed. */
 			shut = 1;
+			goto done;
+		}
 	}
 
 	if (event & EV_WRITE) {
 		n = msgbuf_write(&ibuf->w);
 		if (n == -1 && errno != EAGAIN)
 			fatal("msgbuf_write");
-		if (n == 0)	/* Connection closed. */
+		if (n == 0) {	/* Connection closed. */
 			shut = 1;
+			goto done;
+		}
 	}
 
-	while (err == NULL && check_cancelled(NULL) == NULL) {
+	while (err == NULL) {
+		err = check_cancelled(NULL);
+		if (err)
+			break;
 		if ((n = imsg_get(ibuf, &imsg)) == -1)
 			fatal("%s: imsg_get", __func__);
 		if (n == 0)	/* No more messages. */
@@ -2354,19 +2359,18 @@ repo_write_dispatch(int fd, short event, void *arg)
 			break;
 		default:
 			log_debug("unexpected imsg %d", imsg.hdr.type);
+			err = got_error(GOT_ERR_PRIVSEP_MSG);
 			break;
 		}
 
 		imsg_free(&imsg);
 	}
 
-	if (!shut && check_cancelled(NULL) == NULL) {
-		if (err &&
-		    gotd_imsg_send_error_event(iev, PROC_REPO_WRITE,
-		        client->id, err) == -1) {
-			log_warnx("could not send error to parent: %s",
-			    err->msg);
-		}
+	if (err && gotd_imsg_send_error_event(iev, PROC_REPO_WRITE,
+	    client->id, err) == -1)
+		log_warnx("could not send error to parent: %s", err->msg);
+done:
+	if (!shut) {
 		gotd_imsg_event_add(iev);
 	} else {
 		/* This pipe is dead. Remove its event handler */
