@@ -353,6 +353,14 @@ gotweb_process_request(struct request *c)
 		error = got_get_repo_commits(c, srv->summary_commits_display);
 		if (error)
 			goto err;
+		qs->action = TAGS;
+		error = got_get_repo_tags(c, srv->summary_tags_display);
+		if (error) {
+			log_warnx("%s: got_get_repo_tags: %s", __func__,
+			    error->msg);
+			goto err;
+		}
+		qs->action = SUMMARY;
 		commit = TAILQ_FIRST(&c->t->repo_commits);
 		if (commit && qs->commit == NULL) {
 			qs->commit = strdup(commit->commit_id);
@@ -362,14 +370,6 @@ gotweb_process_request(struct request *c)
 				goto err;
 			}
 		}
-		qs->action = TAGS;
-		error = got_get_repo_tags(c, srv->summary_tags_display);
-		if (error) {
-			log_warnx("%s: got_get_repo_tags: %s", __func__,
-			    error->msg);
-			goto err;
-		}
-		qs->action = SUMMARY;
 		if (gotweb_reply(c, 200, "text/html", NULL) == -1)
 			return;
 		gotweb_render_page(c->tp, gotweb_render_summary);
@@ -380,7 +380,7 @@ gotweb_process_request(struct request *c)
 			log_warnx("%s: %s", __func__, error->msg);
 			goto err;
 		}
-		if (c->t->tag_count == 0) {
+		if (TAILQ_EMPTY(&c->t->repo_tags)) {
 			error = got_error_msg(GOT_ERR_BAD_OBJ_ID,
 			    "bad commit id");
 			goto err;
