@@ -149,7 +149,7 @@ pack_progress(void *arg, int ncolored, int nfound, int ntrees,
 		}
 		return err;
 	}
-	 
+
 	a->ncolored= ncolored;
 	a->nfound = nfound;
 	a->ntrees = ntrees;
@@ -168,6 +168,7 @@ insert_sendable_ref(struct got_pathlist_head *refs, const char *refname,
 	const struct got_error *err;
 	struct got_reference *ref;
 	struct got_object_id *id = NULL;
+	struct got_pathlist_entry *new = NULL;
 	int obj_type;
 
 	err = got_ref_open(&ref, repo, refname, 0);
@@ -191,12 +192,15 @@ insert_sendable_ref(struct got_pathlist_head *refs, const char *refname,
 	case GOT_OBJ_TYPE_TAG:
 		break;
 	default:
-		err = got_error_fmt(GOT_ERR_OBJ_TYPE," cannot send %s",
+		err = got_error_fmt(GOT_ERR_OBJ_TYPE, "cannot send %s",
 		    refname);
 		goto done;
 	}
 
-	err = got_pathlist_insert(NULL, refs, target_refname, id);
+	err = got_pathlist_insert(&new, refs, target_refname, id);
+	if (new == NULL && err == NULL)
+		err = got_error(GOT_ERR_REF_DUP_ENTRY);
+
 done:
 	if (ref)
 		got_ref_close(ref);
@@ -222,7 +226,7 @@ check_common_ancestry(const char *refname, struct got_object_id *my_id,
 		    "bad object type on server for %s", refname);
 
 	err = got_commit_graph_find_youngest_common_ancestor(&yca_id,
-	    my_id, their_id, 0, 0, repo, cancel_cb, cancel_arg);
+	    my_id, their_id, 0, 1, repo, cancel_cb, cancel_arg);
 	if (err)
 		return err;
 	if (yca_id == NULL)
@@ -336,7 +340,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
     struct got_repository *repo, got_send_progress_cb progress_cb,
     void *progress_arg, got_cancel_cb cancel_cb, void *cancel_arg)
 {
-	int imsg_sendfds[2];
+	int imsg_sendfds[2] = { -1, -1 };
 	int npackfd = -1, nsendfd = -1;
 	int sendstatus, done = 0;
 	const struct got_error *err;
@@ -383,8 +387,12 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 			}
 		}
 		err = insert_sendable_ref(&have_refs, branchname, s, repo);
-		if (err)
-			goto done;
+		if (err) {
+			if (err->code != GOT_ERR_REF_DUP_ENTRY)
+				goto done;
+			err = NULL;
+			free(s);
+		}
 		s = NULL;
 	}
 
@@ -419,8 +427,12 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 			}
 		}
 		err = insert_sendable_ref(&have_refs, s, s, repo);
-		if (err)
-			goto done;
+		if (err) {
+			if (err->code != GOT_ERR_REF_DUP_ENTRY)
+				goto done;
+			err = NULL;
+			free(s);
+		}
 		s = NULL;
 	}
 
@@ -450,7 +462,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	if (sendpid == -1) {
 		err = got_error_from_errno("fork");
 		goto done;
-	} else if (sendpid == 0){
+	} else if (sendpid == 0) {
 		got_privsep_exec_child(imsg_sendfds,
 		    GOT_PATH_PROG_SEND_PACK, got_repo_get_path(repo));
 	}
@@ -459,6 +471,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		err = got_error_from_errno("close");
 		goto done;
 	}
+	imsg_sendfds[1] = -1;
 	imsg_init(&sendibuf, imsg_sendfds[0]);
 	nsendfd = dup(sendfd);
 	if (nsendfd == -1) {
@@ -687,6 +700,10 @@ done:
 		if (waitpid(sendpid, &sendstatus, 0) == -1 && err == NULL)
 			err = got_error_from_errno("waitpid");
 	}
+	if (imsg_sendfds[0] != -1 && close(imsg_sendfds[0]) == -1 && err == NULL)
+		err = got_error_from_errno("close");
+	if (imsg_sendfds[1] != -1 && close(imsg_sendfds[1]) == -1 && err == NULL)
+		err = got_error_from_errno("close");
 	if (packfd != -1 && close(packfd) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	if (delta_cache && fclose(delta_cache) == EOF && err == NULL)
