@@ -38,6 +38,7 @@
 
 #include "got_lib_delta.h"
 #include "got_lib_delta_cache.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_object_qid.h"
 #include "got_lib_object_cache.h"
@@ -141,7 +142,7 @@ open_commit(struct got_commit_object **commit, struct got_pack *pack,
 
 	obj->size = len;
 
-	err = got_object_parse_commit(commit, buf, len);
+	err = got_object_parse_commit(commit, buf, len, pack->algo);
 done:
 	got_object_close(obj);
 	free(buf);
@@ -244,7 +245,7 @@ tree_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 		return err;
 
 	err = got_object_parse_tree(entries, nentries, nentries_alloc,
-	    buf, len);
+	    buf, len, id.algo);
 	if (err)
 		goto done;
 
@@ -421,7 +422,7 @@ tag_request(struct imsg *imsg, struct imsgbuf *ibuf, struct got_pack *pack,
 		goto done;
 
 	obj->size = len;
-	err = got_object_parse_tag(&tag, buf, len);
+	err = got_object_parse_tag(&tag, buf, len, id.algo);
 	if (err)
 		goto done;
 
@@ -450,7 +451,7 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 	const struct got_error *err = NULL;
 	struct got_parsed_tree_entry pte1, pte2;
 	const char *seg, *s;
-	size_t seglen;
+	size_t seglen, digest_len;
 	size_t remain1 = *len1, remain2 = *len2, elen;
 	uint8_t *next_entry1 = *buf1;
 	uint8_t *next_entry2 = *buf2;
@@ -460,6 +461,8 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 
 	*changed = 0;
 
+	digest_len = got_hash_digest_length(pack->algo);
+
 	/* We not do support comparing the root path. */
 	if (got_path_is_root_dir(path))
 		return got_error_path(path, GOT_ERR_BAD_PATH);
@@ -490,7 +493,7 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 		 */
 		while (remain1 > 0) {
 			err = got_object_parse_tree_entry(&pte1, &elen,
-			    next_entry1, remain1);
+			    next_entry1, remain1, digest_len, pack->algo);
 			if (err)
 				return err;
 			next_entry1 += elen;
@@ -514,7 +517,7 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 
 		while (remain2 > 0) {
 			err = got_object_parse_tree_entry(&pte2, &elen,
-			    next_entry2, remain2);
+			    next_entry2, remain2, digest_len, pack->algo);
 			if (err)
 				return err;
 			next_entry2 += elen;
@@ -537,7 +540,7 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 			break;
 		}
 
-		if (memcmp(pte1.id, pte2.id, SHA1_DIGEST_LENGTH) == 0) {
+		if (memcmp(pte1.id, pte2.id, pte1.digest_len) == 0) {
 			*changed = 0;
 			break;
 		}
@@ -554,7 +557,8 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 			struct got_object_id id1, id2;
 			int idx;
 
-			memcpy(id1.sha1, pte1.id, SHA1_DIGEST_LENGTH);
+			memcpy(id1.hash, pte1.id, pte1.digest_len);
+			id1.algo = pack->algo;
 			idx = got_packidx_get_object_idx(packidx, &id1);
 			if (idx == -1) {
 				err = got_error_no_obj(&id1);
@@ -570,7 +574,8 @@ tree_path_changed(int *changed, uint8_t **buf1, size_t *len1,
 			next_entry1 = *buf1;
 			remain1 = *len1;
 
-			memcpy(id2.sha1, pte2.id, SHA1_DIGEST_LENGTH);
+			memcpy(id2.hash, pte2.id, pte2.digest_len);
+			id2.algo = pack->algo;
 			idx = got_packidx_get_object_idx(packidx, &id2);
 			if (idx == -1) {
 				err = got_error_no_obj(&id2);
@@ -690,7 +695,7 @@ commit_traversal_request(struct imsg *imsg, struct imsgbuf *ibuf,
 		}
 
 		if (sizeof(struct got_imsg_traversed_commits) +
-		    ncommits * sizeof(commit_ids[0]) >= max_datalen) {
+		    (ncommits + 1) * sizeof(commit_ids[0]) >= max_datalen) {
 			err = send_traversed_commits(commit_ids, ncommits,
 			    ibuf);
 			if (err)
@@ -1144,6 +1149,7 @@ receive_packidx(struct got_packidx **packidx, struct imsgbuf *ibuf)
 	}
 	memcpy(&ipackidx, imsg.data, sizeof(ipackidx));
 
+	p->algo = ipackidx.algo;
 	p->fd = imsg_get_fd(&imsg);
 	p->len = ipackidx.len;
 	if (p->fd == -1) {
@@ -1251,7 +1257,7 @@ enumerate_tree(int *have_all_entries, struct imsgbuf *ibuf, size_t *totlen,
 		}
 
 		err = got_object_parse_tree(&entries, &nentries,
-		    &nentries_alloc, buf, len);
+		    &nentries_alloc, buf, len, pack->algo);
 		if (err)
 			goto done;
 
@@ -1270,7 +1276,8 @@ enumerate_tree(int *have_all_entries, struct imsgbuf *ibuf, size_t *totlen,
 			err = got_object_qid_alloc_partial(&eqid);
 			if (err)
 				goto done;
-			memcpy(eqid->id.sha1, pte->id, sizeof(eqid->id.sha1));
+			eqid->id.algo = pte->algo;
+			memcpy(eqid->id.hash, pte->id, pte->digest_len);
 
 			if (got_object_idset_contains(idset, &eqid->id)) {
 				got_object_qid_free(eqid);
@@ -1379,6 +1386,53 @@ done:
 	return err;
 }
 
+
+static const struct got_error *
+resolve_tag(struct got_object **obj, struct got_object_id *id,
+    struct got_packidx *packidx, struct got_pack *pack,
+    struct got_object_cache *objcache)
+{
+	const struct got_error *err;
+	struct got_object *tagged_obj;
+	struct got_tag_object *tag;
+	uint8_t *buf;
+	size_t len;
+	int idx;
+
+	err = got_packfile_extract_object_to_mem(&buf, &len, *obj, pack);
+	if (err)
+		return err;
+
+	(*obj)->size = len;
+	err = got_object_parse_tag(&tag, buf, len, id->algo);
+	if (err)
+		goto done;
+
+	idx = got_packidx_get_object_idx(packidx, &tag->id);
+	if (idx == -1) {
+		got_object_close(*obj);
+		*obj = NULL;
+		return NULL;
+	}
+
+	tagged_obj = got_object_cache_get(objcache, &tag->id);
+	if (tagged_obj) {
+		tagged_obj->refcnt++;
+	} else {
+		err = open_object(&tagged_obj, pack, packidx,
+		    idx, &tag->id, objcache);
+		if (err)
+			goto done;
+	}
+
+	got_object_close(*obj);
+	*obj = tagged_obj;
+done:
+	got_object_tag_close(tag);
+	free(buf);
+	return err;
+}
+
 static const struct got_error *
 enumeration_request(struct imsg *imsg, struct imsgbuf *ibuf,
     struct got_pack *pack, struct got_packidx *packidx,
@@ -1454,28 +1508,27 @@ enumeration_request(struct imsg *imsg, struct imsgbuf *ibuf,
 		if (err)
 			goto done;
 		if (obj->type == GOT_OBJ_TYPE_TAG) {
-			struct got_tag_object *tag;
-			uint8_t *buf;
-			size_t len;
-			err = got_packfile_extract_object_to_mem(&buf,
-			    &len, obj, pack);
-			if (err)
-				goto done;
-			obj->size = len;
-			err = got_object_parse_tag(&tag, buf, len);
-			if (err) {
-				free(buf);
-				goto done;
+			while (obj->type == GOT_OBJ_TYPE_TAG) {
+				err = resolve_tag(&obj, &qid->id, packidx,
+				    pack, objcache);
+				if (err)
+					goto done;
+				if (obj == NULL)
+					break;
 			}
-			idx = got_packidx_get_object_idx(packidx, &tag->id);
-			if (idx == -1) {
+			if (obj == NULL) {
 				have_all_entries = 0;
 				break;
 			}
+			if (obj->type != GOT_OBJ_TYPE_COMMIT) {
+				got_object_qid_free(qid);
+				qid = NULL;
+				got_object_close(obj);
+				obj = NULL;
+				continue;
+			}
 			err = open_commit(&commit, pack, packidx, idx,
-			    &tag->id, objcache);
-			got_object_tag_close(tag);
-			free(buf);
+			    &obj->id, objcache);
 			if (err)
 				goto done;
 		} else if (obj->type == GOT_OBJ_TYPE_COMMIT) {
@@ -1878,6 +1931,7 @@ receive_pack(struct got_pack **packp, struct imsgbuf *ibuf)
 	}
 	memcpy(&ipack, imsg.data, sizeof(ipack));
 
+	pack->algo = ipack.algo;
 	pack->filesize = ipack.filesize;
 	pack->fd = imsg_get_fd(&imsg);
 	if (pack->fd == -1) {
